import supervision as sv
from groundingdino.util.inference import annotate
def plot_grid(images,idss,boxes,logits,phrases,**args):
  imag_max = args.get("image_max",10) #cambiado, truncado de 20 a 10
  if len(images) > imag_max:
    print(f"Warning: The amount displayed will be truncated to 10. You can change this value using the image_max argument, but there is a risk of not displaying the images correctly.")
    images = images[:20]
    idss = idss[:20]
    boxes = boxes[:20]
    logits = logits[:20]
    phrases = phrases[:20]
  annotated_frames = []
  for i in range(len(boxes)):
    annotated_frame = annotate(image_source=images[i], boxes=boxes[i], logits=logits[i], phrases=phrases[i])
    annotated_frames.append(annotated_frame)
  sv.plot_images_grid(
    images=annotated_frames,
    grid_size=(8, int(len(annotated_frames) / 8)+1),
    size=(15, 15),
    titles=idss
  )

def plot_image(image,boxes,logits,phrases):
  annotated_frame = annotate(image_source=image, boxes=boxes, logits=logits, phrases=phrases)
  sv.plot_image(annotated_frame, (16, 16))


import os 
import gdown
import zipfile

def unzip(file_path,destination_path):
    with zipfile.ZipFile(file_path, 'r') as zip_ref:
        zip_ref.extractall(destination_path)

def download_from_drive(id,destination_path):
    url = f"https://drive.google.com/uc?id={id}&confirm=t"
    if os.path.exists(destination_path):
        return None
    gdown.download(url, destination_path, quiet=False)


def listify(value):
    if not isinstance(value, list):
        value = [value]
    return value 



import wandb
from wandb.integration.keras import WandbCallback
from loss.dice import DiceCoefficient
from Metrics import (jaccard,
                     sensitivity,
                     specificity,
                     )
from Metrics.dice import DiceCoefficientMetric
from .data import get_data
import tensorflow as tf
import numpy as np
import pandas as pd


def calculate_metrics_table(model, metrics, dataset_class, **kwargs_data_augmentation):

    *_, data = get_data(dataset_class, data_augmentation=False,
                        return_label_info=True,
                        **kwargs_data_augmentation)
    data = data.unbatch().batch(1)

    labels = getattr(dataset_class(), "labels_info", None)
    labels = list(labels.keys()) if labels else [None]

    results = {}

    for label in labels:
        if label != None:
            tem_data = data.filter(lambda x, y, l, *id_: l[0] == label)
        else:
            tem_data = data
        for metric in metrics:
            name_metric = metric.name
            curr = []
            for x, y, *_ in tem_data:
                y_pred = model(x)
                curr.append(metric.compute(y, y_pred))
            mean = abs(np.mean(curr))
            std = np.std(curr)
            results.setdefault('metric', []).append(name_metric)
            results.setdefault('label', []).append(label)
            results.setdefault('mean', []).append(mean)
            results.setdefault('std', []).append(std)

    return pd.DataFrame(results)


def get_compile_parameters():
    return {'loss': DiceCoefficient(),
            'optimizer': tf.keras.optimizers.Adam(learning_rate=1e-3),
            'metrics': [jaccard(),
                        sensitivity(),
                        specificity(),
                        DiceCoefficientMetric()
                        ]
            }


def get_train_parameters(dataset_class, data_augmentation=True, validation=True, **kwargs_data_augmentation):
    train_data, val_data, test_data = get_data(
        dataset_class, data_augmentation=data_augmentation, **kwargs_data_augmentation)
    params = {'x': train_data,
              'validation_data': val_data,
              'epochs': 200,
              'callbacks': [WandbCallback(save_model=True)], }
    if not validation:
        params.pop('validation_data')
    return params


def train(model, dataset_class, run=None,
          data_augmentation=True,
          get_compile_parameters=get_compile_parameters,
          get_train_parameters=get_train_parameters,
          validation=True,
          **kwargs_data_augmentation):

    train_parameters = get_train_parameters(dataset_class, data_augmentation=data_augmentation,
                                            validation=validation,
                                            **kwargs_data_augmentation)
    compile_parameters = get_compile_parameters()
    metrics = get_compile_parameters()['metrics']
    model.compile(**compile_parameters)
    model.fit(**train_parameters)
    df_results = calculate_metrics_table(
        model, metrics, dataset_class, **kwargs_data_augmentation)
    if run:
        table = wandb.Table(dataframe=df_results)
        run.log({"metrics": table})



import torch
import numpy as np
class Metrics():
  """
  Class for segmentation metrics
  """
  def __init__(self,smooth=1e-6):
    """
    Args:
      smooth: A float value added to the denominator for numerical stability.
    """
    self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
    self.smooth = torch.tensor(smooth,dtype=torch.float32).to(device=self.device)


  def calculate_metrics(self,y_pred,y_true):
    """
    Calculate the metrics

    Args:
      y_true: True labels
      y_pred: Predicted labels

    Returns:
      A dictionary with the metrics
    """
    y_pred = y_pred.to(device=self.device)
    y_true = y_true.to(device=self.device)
    jaccard = self.jaccard(y_pred,y_true)
    dice = self.dice(y_pred,y_true)
    sensitivity = self.sensitivity(y_pred,y_true)
    specificity = self.specificity(y_pred,y_true)
    return {
        'jaccard':jaccard,
        'dice':dice,
        'sensitivity':sensitivity,
        'specificity':specificity,
    }
  """
  def __call__(self,y_true,y_pred):
    return self.calculate_metrics(y_true,y_pred)
  """

  def calculate_confusion_matrix(self,y_true,y_pred):
    """
    Calculate the confusion matrix

    Args:
      y_true: True labels
      y_pred: Predicted labels

    Returns:
      tp: True positives
      fp: False positives
      fn: False negatives
      tn: True negatives
    """
    output = torch.flatten(y_pred).to(torch.bool)
    target = torch.flatten(y_true).to(torch.bool)
    tp = torch.count_nonzero(output & target).to(torch.float32) # TP
    fp = torch.count_nonzero(output & ~target).to(torch.float32)  # FP
    fn = torch.count_nonzero(~output & target).to(torch.float32)  # FN
    tn = torch.count_nonzero(~output & ~target).to(torch.float32)  # TN
    return tp,fp,fn,tn

  def jaccard(self,y_pred,y_true):
    """
    Calculate the Jaccard or IoU score

    Args:
      y_pred: Predicted labels
      y_true: True labels

    Returns:
      The Jaccard or IoU score
    """
    tp,fp,fn,tn = self.calculate_confusion_matrix(y_pred=y_pred,y_true=y_true)
    return (tp / (tp + fp + fn + self.smooth)).cpu().detach().item()


  def dice(self,y_pred,y_true):
    """
    Calculate the Dice score

    Args:
      y_pred: Predicted labels
      y_true: True labels

    Returns:
      The Dice score
    """
    tp,fp,fn,tn = self.calculate_confusion_matrix(y_pred=y_pred,y_true=y_true)
    return (2 * tp / (2 * tp + fp + fn + self.smooth)).cpu().detach().item()


  def mIoU(self,y_pred,y_true):
    """
    Calculate the mIoU score

    Args:
      y_pred: Predicted labels
      y_true: True labels

    Returns:
      The mIoU score
    """
    tp,fp,fn,tn = self.calculate_confusion_matrix(y_pred=y_pred,y_true=y_true)
    iou_background = tn / (tn + fp + fn + self.smooth)
    iou_object = tp / (tp + fp + fn + self.smooth)
    return ((iou_background + iou_object) / 2.0).cpu().detach().item()


  def sensitivity(self,y_pred,y_true):
    """
    Calculate the sensitivity score

    Args:
      y_pred: Predicted labels
      y_true: True labels

    Returns:
      The sensitivity score
    """
    tp,fp,fn,tn = self.calculate_confusion_matrix(y_pred=y_pred,y_true=y_true)
    return (tp / (tp + fn + self.smooth)).cpu().detach().item()


  def specificity(self,y_pred, y_true):
    """
    Calculate the specificity score

    Args:
      y_pred: Predicted labels
      y_true: True labels

    Returns:
      The specificity score
    """
    tp,fp,fn,tn = self.calculate_confusion_matrix(y_pred=y_pred,y_true=y_true)
    return (tn / (tn + fp + self.smooth)).cpu().detach().item()

  def get_device(self):
    """
    Get the device used by the model.

    Returns:
      The device used by the metrics.
    """
    print(f"Device: {self.device}")
    return self.device


import tensorflow as tf
import numpy as np
import tensorflow_addons as tfa


def resize(shape=(256, 256)):
    def func(img, mask):
        return tf.image.resize(img, shape), tf.image.resize(mask, shape)
    return func


def random_translation(img, mask, translation_h_w):
    shape_img = tf.cast(tf.shape(img), tf.float32)
    dx = tf.cast(shape_img[-2]*translation_h_w[1], tf.int32)
    dy = tf.cast(shape_img[-3]*translation_h_w[0], tf.int32)
    dx = tf.random.uniform(shape=(), minval=-dx, maxval=dx, dtype=tf.int32)
    dy = tf.random.uniform(shape=(), minval=-dy, maxval=dy, dtype=tf.int32)
    img = tfa.image.translate(img, [dx, dy], fill_mode='nearest')
    mask = tfa.image.translate(mask, [dx, dy], fill_mode='constant')
    return img, mask


def random_zoom(img, mask, zoom_h_w):
    img_shape = tf.shape(img)[-3:-1]

    h = tf.cast(img_shape[0], tf.float32)
    h = tf.cast(h*zoom_h_w[0], tf.int32)
    h = tf.random.uniform(shape=(), minval=-h, maxval=h, dtype=tf.int32)

    w = tf.cast(img_shape[1], tf.float32)
    w = tf.cast(w*zoom_h_w[1], tf.int32)
    w = tf.random.uniform(shape=(), minval=-w, maxval=w, dtype=tf.int32)

    img = tf.image.resize_with_crop_or_pad(
        img, img_shape[0]+h,  img_shape[1]+w)
    mask = tf.image.resize_with_crop_or_pad(
        mask, img_shape[0]+h,  img_shape[1]+w)

    img = tf.image.resize(img, img_shape)
    mask = tf.image.resize(mask, img_shape)

    return img, mask


def data_augmentation_func(flip_left_right=True,
                           flip_up_down=True, range_rotate=(-10, 10),
                           translation_h_w=None, zoom_h_w=None):
    def data_aug(img, mask):
        seed = tf.random.uniform(shape=(2,), minval=1,
                                 maxval=1000, dtype=tf.int32)

        if flip_left_right:
            seed = tf.random.uniform(
                shape=(2,), minval=1, maxval=1000, dtype=tf.int32)
            img = tf.image.stateless_random_flip_left_right(img, seed)
            mask = tf.image.stateless_random_flip_left_right(mask, seed)

        if flip_up_down:
            seed = tf.random.uniform(
                shape=(2,), minval=1, maxval=1000, dtype=tf.int32)
            img = tf.image.stateless_random_flip_up_down(img, seed)
            mask = tf.image.stateless_random_flip_up_down(mask, seed)

        if range_rotate:
            min_val = range_rotate[0]/180*np.pi
            max_val = range_rotate[1]/180*np.pi
            rotation = tf.random.uniform(
                shape=(), minval=min_val, maxval=max_val, dtype=tf.float32)
            img = tfa.image.rotate(img, rotation, fill_mode='reflect')
            mask = tfa.image.rotate(mask, rotation, fill_mode='constant')

        if translation_h_w:
            img, mask = random_translation(img, mask, translation_h_w)

        if zoom_h_w:
            img, mask = random_zoom(img, mask, zoom_h_w)

        return img, mask

    return data_aug


def preprocess_data(data, data_augmentation=False,
                    return_label_info=False, shape=256, repeat=1,
                    flip_left_right=True,
                    flip_up_down=True,
                    range_rotate=(-10, 10),
                    translation_h_w=None,
                    zoom_h_w=None):

    if not return_label_info:
        data = data.map(lambda *items: items[:2],
                        num_parallel_calls=tf.data.AUTOTUNE)

    data = data.map(lambda x, y, *l: (*resize((shape, shape))(x, y), *l),
                    num_parallel_calls=tf.data.AUTOTUNE)
    data = data.cache()

    data = data.repeat(repeat)
    if data_augmentation:
        data = data.map(
            data_augmentation_func(
                flip_left_right=flip_left_right,
                flip_up_down=flip_up_down,
                range_rotate=range_rotate,
                translation_h_w=translation_h_w,
                zoom_h_w=zoom_h_w,
            ),
            num_parallel_calls=tf.data.AUTOTUNE
        )

    return data


def get_data(dataset_class, seed=42,
             data_augmentation=True,
             return_label_info=False,
             shape=256, batch_size=32,
             repeat=1,
             flip_left_right=True,
             flip_up_down=True,
             range_rotate=(-10, 10),
             translation_h_w=None,
             zoom_h_w=None,
             split=None):
    options = tf.data.Options()
    options.experimental_distribute.auto_shard_policy = tf.data.experimental.AutoShardPolicy.OFF

    if split:
        dataset = dataset_class(seed=seed, split=split)
    else:
        dataset = dataset_class(seed=seed)
    train_data, val_data, test_data = dataset()

    train_data = preprocess_data(
        train_data, data_augmentation,
        return_label_info, shape, repeat=repeat,
        flip_left_right=flip_left_right,
        flip_up_down=flip_up_down,
        range_rotate=range_rotate,
        translation_h_w=translation_h_w,
        zoom_h_w=zoom_h_w,
    ).batch(batch_size=batch_size).prefetch(tf.data.AUTOTUNE)

    val_data = preprocess_data(
        val_data,
        data_augmentation=False,
        return_label_info=return_label_info,
        shape=shape,
    ).batch(batch_size=batch_size).prefetch(tf.data.AUTOTUNE)

    test_data = preprocess_data(
        test_data,
        data_augmentation=False,
        return_label_info=return_label_info,
        shape=shape,
    ).batch(batch_size=batch_size).prefetch(tf.data.AUTOTUNE)

    train_data = train_data.with_options(options)
    val_data = val_data.with_options(options)
    test_data = test_data.with_options(options)

    return train_data, val_data, test_data



import os
import torch
import numpy as np
from PIL import Image
from typing import List, Union, Tuple, Optional 
from huggingface_hub import hf_hub_download
from segment_anything1.build_sam import sam_model_registry
from segment_anything1.predictor import SamPredictor
from segment_anything1.config import SAM1_MODELS, SAM_NAMES_MODELS
from segment_anything2.sam2_config.config import SAM2_MODELS
from segment_anything2.sam2_image_predictor import SAM2ImagePredictor
from groundingdino.util import box_ops
from groundingdino.util.inference import predict, load_model

class GSamNetwork():
    def __init__(self,SAM: str,SAM_MODEL:Optional[str] = None):
        if not isinstance(SAM, str):
            raise TypeError(f"The SAM parameter should be a single value, not a list or collection. Please provide one of the following valid model names: {SAM_NAMES_MODELS + [None]}.")
        if SAM not in SAM_NAMES_MODELS:
            raise ValueError(f"The specified SAM model '{SAM}' does not exist. Please select a valid model from the following options: {SAM_NAMES_MODELS}.")
        if SAM_MODEL is not None and not isinstance(SAM_MODEL,str):
            raise TypeError(f"The SAM model should be a single value, not a list or collection. Please provide one of the following valid model names: {list(SAM1_MODELS.keys()) + list(SAM2_MODELS.keys()) + [None]}.")
        self.__file_path = os.path.dirname(os.path.abspath(__file__))
        self.default_sam1 = "vit_h"
        self.default_sam2 = "sam2_l"
        self.device = "cuda" if torch.cuda.is_available() else "cpu"

        print("Notice: Loading GroundDINO model")
        self.__Build_GroundingDINO()

        if SAM == "SAM1":
            print("SAM1 selected: a versatile model for object segmentation in images.")
            if SAM_MODEL is None:
                print(f"Warning: No SAM1 model selected. Defaulting to '{self.default_sam1}'.")
                try:
                    self.__Build_SAM1(SAM=self.default_sam1)
                except Exception as e:
                    print(f"An error occurred while loading the SAM1 model '{self.default_sam1}': {str(e)}")
            else:
                if SAM_MODEL not in SAM1_MODELS:
                    raise ValueError(f"The selected SAM model '{SAM_MODEL}' does not exist. Please choose a valid model from the available options: {list(SAM1_MODELS.keys())}.")
                print(f"Notice: Loading the {SAM_MODEL} model")
                try:
                    self.__Build_SAM1(SAM=SAM_MODEL)
                except Exception as e:
                    print(f"An error occurred while loading the SAM1 model '{SAM_MODEL}': {str(e)}")
        
        elif SAM == "SAM2":
            print("SAM2 selected: optimized for images and videos, offering improved object segmentation performance.")
            if SAM_MODEL is None:
                print(f"Warning: No SAM2 model selected. Defaulting to '{self.default_sam2}'")
                try:
                    self.__Build_SAM2(SAM=self.default_sam2)
                except Exception as e:
                    print(f"An error occurred while loading the SAM2 model '{self.default_sam2}': {str(e)}")
            else:
                if SAM_MODEL not in SAM2_MODELS:
                    raise ValueError(f"The selected SAM model '{SAM_MODEL}' does not exist. Please choose a valid model from the available options: {list(SAM2_MODELS.keys())}.")
                print(f"Notice: Loading the {SAM_MODEL} model")
                try:
                    self.__Build_SAM2(SAM=SAM_MODEL)
                except Exception as e:
                    print(f"An error occurred while loading the SAM2 model '{SAM_MODEL}': {str(e)}")


    def __Build_GroundingDINO(self,):
        """
            Build the Grounding DINO model.
        """
        repo_id = "ShilongLiu/GroundingDINO"
        filename = "groundingdino_swint_ogc.pth"
        cache_config = "GroundingDINO_SwinT_OGC.cfg.py"
        try:
            cache_config_file = hf_hub_download(repo_id=repo_id, filename=cache_config)
            pth_file = hf_hub_download(repo_id=repo_id, filename=filename)
        except:
            raise RuntimeError(f"Error downloading GroundingDINO model. Please ensure that the {repo_id}/{cache_config} file exists in huggingface_hub and the {filename} checkpoint is functional.")
        
        try:
            self.groundingdino = load_model(cache_config_file,pth_file, device=self.device)
        except Exception as e:
            raise RuntimeError(f"An error occurred while loading the GroundingDINO model: {str(e)}")


    
    def __Build_SAM1(self,
                     SAM:str,) -> None:
        """
            Build the SAM1 model.

            Args:
                SAM: The name of the SAM model to build.
        """
        try:
            checkpoint_url = SAM1_MODELS[SAM]
            sam = sam_model_registry[SAM]()
            state_dict = torch.hub.load_state_dict_from_url(checkpoint_url)
        except Exception as e:
            raise RuntimeError(f"Error downloading SAM1. Please ensure the model is correct: {SAM} and that the checkpoint is functional: {checkpoint_url}.")
        try:
            sam.load_state_dict(state_dict, strict=True)
            sam.to(device=self.device)
            self.SAM1 = SamPredictor(sam)
        except Exception as e:
            raise RuntimeError(f"SAM1 model can't be compile: {str(e)}")




    def __Build_SAM2(self,
                     SAM:str) -> None:
        """
            Build the SAM2 model.

            Args:
                SAM: The name of the SAM model to build.
        """
        try:
            checkpoint_url = SAM2_MODELS[SAM]
            self.SAM2 = SAM2ImagePredictor.from_pretrained(checkpoint_url)
        except:
            raise RuntimeError(f"Error downloading or Compile {SAM} model. Please ensure that {SAM2_MODELS[SAM]} is functional.")

    def predict_dino(self, 
                     image: Union[Image.Image, 
                                  torch.Tensor, 
                                  np.ndarray], 
                     text_prompt: str, 
                     box_threshold: float, 
                     text_threshold: float,
                     box_process_threshold: float,
                     Normalize:bool = False,) -> torch.Tensor:
        """
            Run the Grounding DINO model for bounding box prediction.

            Args:
                image: The input image with (WxHxC) shape.
                text_prompt: The text prompt for bounding box prediction.
                box_threshold: The threshold for bounding box prediction.
                text_threshold: The threshold for text prediction.
                Normalize (optional): Whether to normalize the image. Defaults to False.

            Returns:
                The predicted bounding boxes with (B,4) shape with logits and phrases.
        """
        
        image_trans = load_image(image)
        image_array = convert_image_to_numpy(image)
        shape =  image_array.shape[:2]
        boxes, logits, phrases = predict(model=self.groundingdino,
                                         image=image_trans,
                                         caption=text_prompt,
                                         box_threshold=box_threshold,
                                         text_threshold=text_threshold,
                                         device=self.device)
        if Normalize:
            W,H = shape
            boxes = box_ops.box_cxcywh_to_xyxy(boxes) * torch.Tensor([W, H, W, H])

        boxes,logits,phrases = PostProcessor().postprocess_box(image_shape=shape,
                                                             threshold=box_process_threshold,
                                                             boxes_list=boxes,
                                                             logits_list=logits,
                                                             phrases_list=phrases,
                                                             mode="single")
        return boxes, logits, phrases
    
    def predict_dino_batch(self,
                           images:List[Union[Image.Image,torch.Tensor,np.ndarray]],
                           text_prompt: str, 
                           box_threshold: float, 
                           text_threshold: float,
                           box_process_threshold: float,
                           Normalize:bool = False,) -> Tuple[List[torch.Tensor],List[torch.Tensor],List[torch.Tensor]]:
        """
            Run the Grounding DINO model for batch prediction.

            Args:
                images: The input images with (WxHxC) shape.
                text_prompt: The text prompt for bounding box prediction.
                box_threshold: The threshold for bounding box prediction.
                text_threshold: The threshold for text prediction.
                Normalize (optional): Whether to normalize the image. Defaults to False

            Returns:
                The predicted bounding boxes with (B,4) shape with logits and phrases.
        """
        results = list(map(lambda image: self.predict_dino(image=image,
                                                           text_prompt=text_prompt,
                                                           box_threshold=box_threshold,
                                                           text_threshold=text_threshold,
                                                           box_process_threshold=box_process_threshold,
                                                           Normalize=Normalize), images))
        boxes, logits, phrases = zip(*results)
        boxes = list(boxes)
        logits = list(logits)
        phrases = list(phrases)
        return boxes, logits, phrases
    
    def predict_SAM1(self,
                    image: Union[Image.Image, 
                                 torch.Tensor,
                                 np.ndarray], 
                    area_thresh: float,
                    boxes: Optional[torch.Tensor] = None,
                    points_coords: Optional[torch.Tensor] = None,
                    points_labels: Optional[torch.Tensor] = None) -> torch.Tensor:
        """
            Run the SAM1 model for image segmentation.

            Args:
                image: The input image with (WxHxC) shape.
                boxes: The bounding boxes for segmentation. Defaults to None.
                points_coords: The coordinates of the points for segmentation. Defaults to None.
                points_labels: The labels of the points for segmentation. Defaults to None.

            Returns
                The predicted segmentation mask with (WxHx1) shape.
    """
        image_array = convert_image_to_numpy(image)
        transformed_boxes,transformed_points,points_labels = self.__prep_prompts(boxes,
                                                                                 points_coords,
                                                                                 points_labels,
                                                                                 image_array.shape[:2])

        self.SAM1.set_image(image_array)
        masks, _, _ = self.SAM1.predict_torch(point_coords=transformed_points.to(self.SAM1.device) if transformed_points is not None else None,
                                              point_labels=points_labels.to(self.SAM1.device) if points_labels is not None else None,
                                              boxes=transformed_boxes.to(self.SAM1.device) if transformed_boxes is not None else None,
                                              multimask_output=False,)
        self.SAM1.reset_image()
        masks = PostProcessor().postprocess_masks(masks=masks,
                                                area_thresh=area_thresh)
        masks = masks.cpu()
        mask = torch.any(masks,dim=0).permute(1,2,0).numpy()
        return mask
    
    def predict_SAM1_batch(self,
                           images:List[Union[Image.Image,
                                             torch.Tensor,
                                             np.ndarray]],
                            area_thresh: float,
                           boxes:Optional[List[torch.Tensor]] = None,
                           points_coords:Optional[List[torch.Tensor]] = None,
                           points_labels:Optional[List[torch.Tensor]] = None) -> List[torch.Tensor]:
        """
            Run the SAM1 model for batch prediction.

            Args:
                images: The input images with (WxHxC) shape.
                boxes: List of bounding boxes for each image. Can be None.
                points_coords: List of point coordinates for each image. Can be None.
                points_labels: List of point labels for each image. Can be None.

            Returns:
                The predicted masks for each image.
        """
        if points_coords is not None and points_labels is None:
            raise ValueError("If 'points_coords' is provided, 'points_labels' must also be provided, and vice versa.")
        elif points_labels is not None and points_coords is None:
            raise ValueError("If 'points_labels' is provided, 'points_coords' must also be provided, and vice versa.")
        
        if boxes is None:
            boxes = [None] * len(images)
        if points_coords is None:
            points_coords = [None] * len(images)
            points_labels = [None] * len(images)

        if not (len(images) == len(boxes) == len(points_coords) == len(points_labels)):
            raise ValueError("The lengths of 'images', 'boxes', 'points_coords', and 'points_labels' must match.")
        
        def process_image(image: Union[Image.Image,torch.Tensor,np.ndarray],
                          box: Optional[torch.Tensor],
                          point_coord: Optional[torch.Tensor],
                          point_label: Optional[torch.Tensor]) -> torch.Tensor:
            """
                Process a single image with its corresponding boxes and points.

                Args:
                    image: The input image with (WxHxC) shape.
                    box: The bounding boxes for the image.
                    point_coords: The point coordinates for the image.
                    point_labels: The point labels for the image.

                Returns:
                    np.ndarray: The predicted mask for the image.
            """
            mask = self.predict_SAM1(image=image,
                                     area_thresh=area_thresh,
                                     boxes=box,
                                     points_coords=point_coord,
                                     points_labels=point_label)
            return mask
        results = [process_image(image, box, point_coords, point_labels) for image, box, point_coords, point_labels in zip(images, boxes, points_coords, points_labels)]
        return results
    def predict_SAM2(self,
                     image: Union[Image.Image,
                                  torch.Tensor,
                                  np.ndarray],
                     point_coords: np.ndarray,
                     point_labels: np.ndarray,
                     box: np.ndarray, 
                     multimask_output: bool = False) -> np.ndarray:
        
        """
            
        """
        image_array = convert_image_to_numpy(image)
        with torch.inference_mode(),  torch.autocast("cuda", dtype=torch.bfloat16):
            self.SAM2.set_image(image_array)
            masks,_,_ = self.SAM2.predict(point_coords=point_coords,
                              point_labels=point_labels,
                              box=box,
                              multimask_output=multimask_output)

            self.SAM2.reset_predictor()
        
        return masks

    def predict_SAM2_batch(self,
                           images_array: List[Union[Image.Image,torch.Tensor,np.ndarray]],
                           points_coords: List[Union[np.ndarray]],
                           points_labels: List[Union[np.ndarray]],
                           boxes: List[Union[np.ndarray]],
                           multimask_output: bool = False):
            images_numpy_array = [convert_image_to_numpy(image) for image in images_array]
            with torch.inference_mode(),  torch.autocast("cuda", dtype=torch.bfloat16):
                self.SAM2.set_image_batch(images_numpy_array)
                masks,_,_ = self.SAM2.predict_batch(point_coords=points_coords,
                                                    point_labels=points_labels,
                                                    box=boxes,
                                                    multimask_output=multimask_output)
                self.SAM2.reset_predictor()
            return masks
    
    def reset_model_SAM1(self):
        self.SAM1.reset_image()
    
    def reset_model_SAM2(self):
        self.SAM2.reset_predictor()

    
    def __prep_prompts(self,boxes,points_coords,points_labels,dims):
        H,W = dims #Cambiado dimensiones al reves
        if boxes is not None:
            clip_valor = np.clip(boxes[0][0], 0, 1)
            if clip_valor == boxes[0][0]:
                boxes = box_ops.box_cxcywh_to_xyxy(boxes) * torch.Tensor([W,H,W,H])
                transformed_boxes = self.SAM1.transform.apply_boxes_torch(boxes, (W,H))
            else:
                transformed_boxes = boxes #Agregado 
        else:
            transformed_boxes=None

        if points_coords is not None and points_labels is None:
            raise ValueError("If 'points_coords' is provided, 'points_labels' must also be provided, and vice versa.")
        elif points_labels is not None and points_coords is None:
            raise ValueError("If 'points_labels' is provided, 'points_coords' must also be provided, and vice versa.")
        elif points_coords is not None and points_labels is not None:
            transformed_points = self.SAM1.transform.apply_coords_torch(points_coords, (W,H))
        else:
             transformed_points = None
             points_labels = None
    
        return transformed_boxes,transformed_points,points_labels
        
        
        
if __name__ == "__main__":
    from groundino_samnet.utils import PostProcessor, load_image, convert_image_to_numpy
    SAM1 = GSamNetwork(SAM="SAM2",SAM_MODEL="sa")
else:
    from .utils import PostProcessor, load_image, convert_image_to_numpy


import os
import glob
import sys  
os.environ['TORCH_CUDA_ARCH_LIST'] = '8.0'
import subprocess
import torch
from torch.utils.cpp_extension import CUDA_HOME,CppExtension,CUDAExtension,BuildExtension
from setuptools.command.install import install
import pathlib

from setuptools import setup, find_packages
link = "https://download.pytorch.org/whl/cu121"
REQUIRED_PACKAGES = [
    "numpy==1.26.4",
    "transformers==4.42.4",
    "huggingface_hub==0.23.5",
    "addict==2.4.0",
    "opencv-python==4.10.0.84",
    "pycocotools",
    "yapf",
    "timm",
    "supervision==0.22.0",
    "tqdm>=4.66.1",
    "scikit-learn",
    "hydra-core>=1.3.2",
    "iopath>=0.1.10",
    "ninja",
    "kaggle"
]

def get_extensions():
    this_dir = os.path.dirname(os.path.abspath(__file__))
    extensions_dir = os.path.join(this_dir, "src", "groundingdino", "models", "GroundingDINO", "csrc")
    extension_dir_sam = os.path.join(this_dir,"src","segment_anything2","csrc")
    srcs_sam2 = [f"{this_dir}/src/segment_anything2/csrc/connected_components.cu"]

    main_source = os.path.join(extensions_dir, "vision.cpp")
    sources = glob.glob(os.path.join(extensions_dir, "**", "*.cpp"))
    source_cuda = glob.glob(os.path.join(extensions_dir, "**", "*.cu")) + glob.glob(
        os.path.join(extensions_dir, "*.cu")
    )
    sources = [main_source] + sources

    extension = CppExtension

    extra_compile_args = {"cxx": []}
    define_macros = []

    if CUDA_HOME is not None and (torch.cuda.is_available() or "TORCH_CUDA_ARCH_LIST" in os.environ):
        print("Compiling with CUDA")
        extension = CUDAExtension
        sources += source_cuda

        define_macros += [("WITH_CUDA", None)]
        extra_compile_args["nvcc"] = [
            "-DCUDA_HAS_FP16=1",
            "-D__CUDA_NO_HALF_OPERATORS__",
            "-D__CUDA_NO_HALF_CONVERSIONS__",
            "-D__CUDA_NO_HALF2_OPERATORS__",
            "-allow-unsupported-compiler"
        ]

    else:
        print("Compiling without CUDA")
        define_macros += [("WITH_HIP", None)]
        extra_compile_args["nvcc"] = []
        return None

    sources = [os.path.join(extensions_dir, s) for s in sources]
    include_dirs = [extensions_dir] 

    ext_modules = [
        extension(
            "groundingdino._C",
            sources,
            include_dirs=include_dirs,
            define_macros=define_macros,
            extra_compile_args=extra_compile_args,
        ),
        extension(
            "segment_anything2._C", 
            sources=srcs_sam2, 
            include_dirs=[extension_dir_sam],
            define_macros=define_macros,
            extra_compile_args=extra_compile_args
        )
    ]

    return ext_modules

def build_extensions():


    with open("LICENSE", "r", encoding="utf-8") as f:
        license = f.read()

    HERE = pathlib.Path(__file__).parent
    README = (HERE / "description.md").read_text()
    setup(
        name="groundino_samnet",
        version="0.1.17",
        author="Wilhelm David Buitrago Garcia",
        url="https://github.com/WilhelmBuitrago/DiagAssistAI",
        description="A SAM model with GroundingDINO model",
        long_description=README,
        long_description_content_type="text/markdown",  # Este especifica el tipo de contenido del long_description.
        license=license,
        package_dir={"": "src"},
        packages=find_packages(where="src"),
        install_requires=REQUIRED_PACKAGES,
        ext_modules=get_extensions(),
        cmdclass={"build_ext": BuildExtension},
        python_requires='==3.10.12',
        classifiers=[
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.10"],
        project_urls={
        "Grounding-samnet": "https://github.com/WilhelmBuitrago/DiagAssistAI"
        },
    )

if __name__ == "__main__":
    build_extensions()


"""
===============================
Infrared Thermal Images of Feet
===============================
"""

import os
from glob import glob
import cv2
import numpy as np
import tensorflow as tf
from sklearn.model_selection import GroupShuffleSplit


class InfraredThermalFeet:
    already_unzipped = False

    def __init__(self, split=[0.2, 0.2], seed: int = 42,
                 id_: str = "1HZa4pVwlIXCrRIidflB158kmtYGW23Qe"):
        options = tf.data.Options()
        options.experimental_distribute.auto_shard_policy = tf.data.experimental.AutoShardPolicy.DATA

        self.split = listify(split)
        self.seed = seed

        self.__id = id_
        self.__folder = os.path.join(os.path.dirname(__file__),
                                     'InfraredThermalFeet')
        self.__path_images = os.path.join(self.__folder,
                                          'dataset' + "\\")

        if not InfraredThermalFeet.already_unzipped:
            self.__set_env()
            InfraredThermalFeet.already_unzipped = True

        self.file_images = glob(os.path.join(
            self.__path_images, '*[!(mask)].jpg'))
        self.file_images = list(map(lambda x: x[:-4], self.file_images))
        self.file_images = sorted(self.file_images)

        self.groups = list(map(lambda x: os.path.split(x)[-1].split('_')[0],
                               self.file_images))

        self.num_samples = len(self.file_images)

    def __set_env(self):
        destination_path_zip = os.path.join(self.__folder,
                                            'InfraredThermalFeet.zip')
        os.makedirs(self.__folder, exist_ok=True)
        download_from_drive(self.__id, destination_path_zip)
        unzip(destination_path_zip, self.__folder)

    @staticmethod
    def __preprocessing_mask(mask):
        mask = mask[..., 0] > 0.5
        mask = mask.astype(np.float32)
        return mask[..., None]

    def load_instance_by_id(self, id_img):
        root_name = os.path.join(self.__path_images, id_img)
        return self.load_instance(root_name)

    @staticmethod
    def load_instance(root_name):
        img = cv2.imread(f'{root_name}.jpg')/255
        img = img[..., 0][..., None]
        mask = cv2.imread(f'{root_name}_mask.png')
        mask = InfraredThermalFeet.__preprocessing_mask(mask)
        id_image = os.path.split(root_name)[-1]
        return img, mask, id_image

    @staticmethod
    def __gen_dataset(file_images):
        def generator():
            for root_name in file_images:
                yield InfraredThermalFeet.load_instance(root_name)
        return generator

    def __generate_tf_data(self, files):
        output_signature = (tf.TensorSpec((None, None, None), tf.float32),
                            tf.TensorSpec((None, None, None), tf.float32),
                            tf.TensorSpec(None, tf.string),
                            )

        dataset = tf.data.Dataset.from_generator(self.__gen_dataset(files),
                                                 output_signature=output_signature)

        len_files = len(files)
        dataset = dataset.apply(
            tf.data.experimental.assert_cardinality(len_files))
        return dataset

    def __get_log_tf_data(self, i, files):
        print(f' Number of images for Partition {i}: {len(files)}')
        return self.__generate_tf_data(files)

    @staticmethod
    def __shuffle(X, seed):
        np.random.seed(seed)
        np.random.shuffle(X)

    @staticmethod
    def _train_test_split(X, groups, random_state=42, test_size=0.2):
        gss = GroupShuffleSplit(n_splits=1, test_size=test_size,
                                random_state=random_state)
        indxs_train, index_test = next(gss.split(X, groups=groups))

        InfraredThermalFeet.__shuffle(indxs_train, random_state)
        InfraredThermalFeet.__shuffle(index_test, random_state)

        return X[indxs_train], X[index_test], groups[indxs_train], groups[index_test]

    def __call__(self,):
        file_images = np.array(self.file_images)
        groups = np.array(self.groups)
        train_imgs, test_imgs, g_train, _ = InfraredThermalFeet._train_test_split(
            file_images,
            groups,
            test_size=self.split[0],
            random_state=self.seed
        )

        train_imgs, val_imgs, _, _ = InfraredThermalFeet._train_test_split(
            train_imgs, g_train,
            test_size=self.split[1],
            random_state=self.seed)

        p_files = [train_imgs, val_imgs, test_imgs]

        partitions = [self.__get_log_tf_data(
            i+1, p) for i, p in enumerate(p_files)]

        return partitions


class ModifiedInfraredThermalFeet(InfraredThermalFeet):
    def __init__(self, split=[0.2, 0.2], seed: int = 42,
                 file_path: str = "InfraredThermalFeet.zip",
                 *args, **kwargs):

        self.split = listify(split)
        self.seed = seed

        self.file_path = file_path
        self.__folder = os.path.join(os.path.dirname(__file__),
                                     'InfraredThermalFeet')
        self.__path_images = os.path.join(self.__folder,
                                          'dataset' + "\\")

        if not InfraredThermalFeet.already_unzipped:
            self.__set_env()
            InfraredThermalFeet.already_unzipped = True

        self.file_images = glob(os.path.join(
            self.__path_images, '*[!(mask)].jpg'))
        self.file_images = list(map(lambda x: x[:-4], self.file_images))
        self.file_images = sorted(self.file_images)

        self.groups = list(map(lambda x: os.path.split(x)[-1].split('_')[0],
                               self.file_images))
        self.num_samples = len(self.file_images)

    def __set_env(self):
        path_zip = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'DataSet',
                                'InfraredThermalFeet.zip')
        os.makedirs(self.__folder, exist_ok=True)
        unzip(path_zip, self.__folder)


if __name__ == "__main__":
    import convRFFds.data as data
    from utils import download_from_drive
    from utils import unzip
    from utils import listify
    kwargs_data_augmentation = dict(repeat=1,
                                    batch_size=2,
                                    shape=224,
                                    split=[0.4, 0.3]
                                    )
    dataset = ModifiedInfraredThermalFeet
    train_dataset, val_dataset, test_dataset = data.get_data(
        dataset_class=dataset, data_augmentation=False, return_label_info=True, **kwargs_data_augmentation)
else:
    from .utils import download_from_drive
    from .utils import unzip
    from .utils import listify
